﻿using Plane.Copters;
using GalaSoft.MvvmLight;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Plane.FormationCreator.Util;
using System.Windows.Input;
using GalaSoft.MvvmLight.Command;
using Microsoft.Practices.ServiceLocation;

namespace Plane.FormationCreator.Formation
{
    public class GroupManager: ObservableObject
    {
        public ObservableCollection<CopterGroup> copterGroups { get; } = new ObservableCollection<CopterGroup>();
        private CopterManager _copterManager = ServiceLocator.Current.GetInstance<CopterManager>();
        private string _LastSelectedGroup;
        public string LastSelectedGroup
        {
            get { return _LastSelectedGroup; }
            set { Set(nameof(LastSelectedGroup), ref _LastSelectedGroup, value); }
        }
        public GroupManager()
        {
        }

        public void AddGroup(string name = "")
        {
            CopterGroup group = new CopterGroup(name);
            copterGroups.Add(group);
            group.RemoveGroupEvent += new CopterGroup.RemoveGroupEventHandler(GroupRemove);
            group.SelectedGroupEvent += new CopterGroup.SelectedGroupEventHandler(SelectedGroup);
        }

        public void SelectedGroup(object sender)
        {
            CopterGroup groups = sender as CopterGroup;
            LastSelectedGroup = groups.groupName;
        }

        public void GroupRemove(object sender)
        {
            CopterGroup groups = sender as CopterGroup;
            copterGroups.Remove(groups);
        }


        public IEnumerable<object> ExportGroups()
        {
            return copterGroups;
        }

        public void ImportGroupsInfo(dynamic groupsInfo)
        {
            copterGroups.Clear();
            foreach (var group in groupsInfo)
            {
                string name = group.groupName;
                CopterGroup copterGroup = new CopterGroup(name);
                foreach (var index in group.copterIndexList)
                {
                    copterGroup.copterIndexList.Add(Convert.ToInt32(index));
                }
                copterGroups.Add(copterGroup);
                copterGroup.RemoveGroupEvent += new CopterGroup.RemoveGroupEventHandler(GroupRemove);
                copterGroup.SelectedGroupEvent += new CopterGroup.SelectedGroupEventHandler(SelectedGroup);
            }
        }


        private ICommand _AddGroupsCommand;
        public ICommand AddGroupsCommand
        {
            get
            {
                return _AddGroupsCommand ?? (_AddGroupsCommand = new RelayCommand<double>(async =>
                {
                    AddGroup();
                }));
            }
        }



    }

    public class CopterGroup : ObservableObject
    {
        private CopterManager _copterManager = ServiceLocator.Current.GetInstance<CopterManager>();
        public CopterGroup(string name = "")
        {
            groupName = name != "" ? name : "新分组";
            
        }
        public string groupName { get; set; }
        public List<int> copterIndexList { get; } = new List<int>();

        public delegate void RemoveGroupEventHandler(object sender);
        public event RemoveGroupEventHandler RemoveGroupEvent;
        private ICommand _RemoveGroupCommand;
        [Newtonsoft.Json.JsonIgnore]
        public ICommand RemoveGroupCommand
        {
            get
            {
                return _RemoveGroupCommand ?? (_RemoveGroupCommand = new RelayCommand<double>(async =>
                {
                    RemoveGroupEvent(this);
                }));
            }
        }


        public delegate void SelectedGroupEventHandler(object sender);
        public event SelectedGroupEventHandler SelectedGroupEvent;
        private ICommand _SelectCoptersCommand;
        [Newtonsoft.Json.JsonIgnore]
        public ICommand SelectCoptersCommand
        {
            get
            {
                return _SelectCoptersCommand ?? (_SelectCoptersCommand = new RelayCommand<double>(async =>
                {
                    _copterManager.Select(null);
                    _copterManager.shiftkeydown = true;
                    foreach (var copterIndex in copterIndexList)
                    {
                        //if (_copterManager.Copters.Count > copterIndex)
                       // {
                            //_copterManager.Select(_copterManager.Copters[copterIndex]);
                            ICopter copter = null;
//                             try
//                             {
//                                 copter = _copterManager.Copters.First(o => o.Name == (copterIndex + 1).ToString());
//                             }
//                             catch (Exception)
//                             {
//                                 continue;
//                             }
                            copter = _copterManager.Copters.FirstOrDefault(o => o.Name == (copterIndex + 1).ToString());
                            if (copter != null)
                            {
                                _copterManager.Select(copter);
                            }
                        //}
                    }
                    _copterManager.shiftkeydown = false;
                    SelectedGroupEvent(this);
                }));
            }
        }

        private ICommand _SetGroupCoptersCommand;
        [Newtonsoft.Json.JsonIgnore]
        public ICommand SetGroupCoptersCommand
        {
            get
            {
                return _SetGroupCoptersCommand ?? (_SetGroupCoptersCommand = new RelayCommand<double>(async =>
                {
                    copterIndexList.Clear();
                    foreach (var copter in _copterManager.SelectedCopters)
                    {
                        copterIndexList.Add(_copterManager.Copters.IndexOf(copter));
                    }
                }));
            }
        }
    }
}
